<?php
/**
  * @author    United Parcel Service of America, Inc.*
  * @copyright (c) 2019 United Parcel Service of America, Inc., all rights reserved*
  * @license   This work is Licensed under the Academic Free License version 3.0
  *            http://opensource.org/licenses/afl-3.0.php *
  * @link      https://www.ups.com/pl/en/services/technology-integration/ecommerce-plugins.page *
 */

require_once dirname(__FILE__) . '/../../common/CommonController.php';

class AdminUpsBillPrefController extends CommonController
{
    private $links = array();

    public function __construct()
    {
        $this->bootstrap = true;
        parent::__construct();

        $result = CommonFunction::checkScreenConfig();
        if ($result !== 'DONE'
            && $result !== Tools::getValue('controller')) {
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminUpsBillPref'));
        }

        $this->links = $this->getInformationLink();

        $this->fields_options = $this->createFieldsOption();
    }

    public function setMedia($isNewTheme = false)
    {
        parent::setMedia($isNewTheme);
        if (Tools::version_compare(_PS_VERSION_, '8.0.0', '>=')) {
            $this->addJs($this->module->getPathUri() . '/views/js/button.js');
            $this->addCSS($this->module->getPathUri().'/views/css/upsmodule.css');
        }else{
            $this->addCSS(_PS_MODULE_DIR_ . $this->module->name . '/views/css/upsmodule.css');
            $this->addJS(_PS_MODULE_DIR_ . $this->module->name . '/views/js/button.js');
        }
    }

    public function initToolBarTitle()
    {
        $this->toolbar_title[] = $this->l('AdminUpsBillPref');
    }

    private function createFieldsOption()
    {
        return array(
            'indexation' => array(
                'title'  => $this->sdk->t('billing', 'ttlBilling'),
                'icon'   => 'icon-credit-card',
                'info'   => $this->drawHtml(),
                'submit' => array(
                    'title'    => $this->sdk->t('button', 'txtComplete'),
                    'imgclass' => 'ok',
                    'name'     => 'submitConfiguration',
                    'id' => 'btnCompleted',
                ),
            ),
        );
    }

    private function drawHtml()
    {
        $sdk = $this->sdk;
        $getLinkDownloadRegis = Context::getContext()->link->getAdminLink('AdminUpsBillPref') . '&downloadRegis';
        $getLinkDownloadCod   = Context::getContext()->link->getAdminLink('AdminUpsBillPref') . '&downloadCod';

        $linkIndividual = $this->links['individual_shipper'];
        $linkPickupSchedule = $this->links['pickup_schedule'];
        $urlUPSsearch = Constants::SEARCH_ADDRESS_POINT_URL;

        $keys = array();
        if ($this->module->usa()) {
            $keys['txtPickupContent2'] = 'txtPickupContent2US';
            $keys['txtPickupContentSub2'] = 'txtPickupContentSub2US';
            $keys['txtPickupElemnent3'] = 'txtPickupElemnent3US';
            $keys['txtPickupElemnentSub3'] = 'txtPickupElemnentSub3US';
            $keys['txtInfo'] = 'txtInfoUS';
            $keys['txtPickupElement4'] = 'txtPickupElement4US';
            $keys['txtSearchAP'] = 'txtSearchAPUS';
        } else {
            $keys['txtPickupContent2'] = 'txtPickupContent2';
            $keys['txtPickupContentSub2'] = 'txtPickupContentSub2';
            $keys['txtPickupElemnent3'] = 'txtPickupElemnent3';
            $keys['txtPickupElemnentSub3'] = 'txtPickupElemnentSub3';
            $keys['txtInfo'] = 'txtInfo_NotForPL';
            $keys['txtPickupElement4'] = 'txtPickupElement4_NotForPL_sub';
            $keys['txtSearchAP'] = 'txtSearchAP';
        }

        $titleInfo  = $sdk->t('billing', 'txtStart');
        $desc       = $sdk->t('billing', 'txtCompleteDesc');
        $ttlFQAs    = $sdk->t('billing', 'txtFAQs');
        $content1   = $sdk->t('billing', 'txtPickupContent');
        $content1_sub   = $sdk->t('billing', 'txtPickupContentSub');
        $content2       = $sdk->t('billing', $keys['txtPickupContent2']);
        $content2_sub   = $sdk->t('billing', $keys['txtPickupContentSub2']);
        $content3       = $sdk->t('billing', $keys['txtPickupElemnent3']);
        $content3_sub   = $sdk->t('billing', $keys['txtPickupElemnentSub3']);
        $ttlNegotiatedRates    = $sdk->t('billing', 'txtNegotiatedRates');
        $contentNegotiatedRatesSub   = $sdk->t('billing', 'txtNegotiatedRatesSub');
        $contentUserId   = $sdk->t('billing', 'txtUserId').'['.Configuration::get('MY_UPS_ID').'].';
       
        $contentUserIdSub   = $sdk->t('billing', 'txtUserIdSub').'['.Configuration::get('MY_UPS_ID').'].'. $sdk->t('billing', 'txtUserIdSub2') ;

        $txtSearch      = $sdk->t('ups', $keys['txtSearchAP']);

        $html = "<h5 class='content-bold'>$titleInfo</h5>
        <p class='pad-paragraph'>$desc</p>
        <h5 class='content-bold'>$ttlNegotiatedRates</h5>
        <p class='pad-paragraph'>$contentNegotiatedRatesSub</p>
        <p class='pad-paragraph'>$contentUserId</p>
        <p class='pad-paragraph'>$contentUserIdSub</p>
        <h5 class='content-bold'>$ttlFQAs</h5>
        <p class='pad-paragraph'>$content1</p>
        <p class='pad-paragraph'>$content1_sub</p>
        <p class='pad-paragraph'>$content2</p>
        <p class='pad-paragraph'>$content2_sub<a target='_blank' href='$linkIndividual'>$linkIndividual</a></p>
        <p class='pad-paragraph'>$content3</p>
        <p class='pad-paragraph'>$content3_sub</p>
        <p class='pad-paragraph'>
            <a target='_blank' href='$urlUPSsearch' class='btn btn-default'>$txtSearch</a></p>
        <p class='pad-paragraph'>"
        . $sdk->t('billing', 'txtInfo')
        . "<a target='_blank' href='$linkPickupSchedule'>$linkPickupSchedule</a><br><br>"
        . $this->getScheduleTexts()
        . '</p>'
        . $this->generateBtnPrintForm($getLinkDownloadRegis)
        . '<p class="pad-paragraph">' . $this->getFQAlinks() . '</p>'
        . $this->generateBtnPrintForm($getLinkDownloadCod);

        return $html;
    }

    public function initProcess()
    {
        if (Tools::getIsset('downloadRegis')) {
            $this->action = 'downloadRegis';
        }

        if (Tools::getIsset('downloadCod')) {
            $this->action = 'downloadCod';
        }
    }

    public function postProcess()
    {
        // $orders_to_mig = $this->getOrdersToMigrate();
        // if(!empty($orders_to_mig)){
        //     foreach($orders_to_mig as $order){
        //         $this->migrateOldOrder($order['id']);
        //     }
        // }
        if (Tools::isSubmit('submitConfiguration')) {
            CommonFunction::setDoneConfigScreen(Tools::getValue('controller'));
            if (Configuration::get('UPS_CONFIG_DONE') && Configuration::get('UPS_SHIPING_METHOD_ID') == -1) {
                $this->createCarrier(); // Create UPS shipping carrier
                if (Tools::version_compare(_PS_VERSION_, '1.7.4', '>=')) {
                    InitFunction::enableTab('AdminUpsAccountSuccess');
                    InitFunction::disableTab('AdminUpsAccount');
                }
            }
             
            Tools::redirectAdmin(Context::getContext()->link->getAdminLink('AdminUpsOpenOrders'));
        }
        parent::postProcess();
    }
    //Migrate old orders into shipment manager
    public function migrateOldOrder($order_id)
    {
        $order = new Order((int) $order_id); 
        $products = $order->getProducts();
        $dimensionUnit = Configuration::get('PS_DIMENSION_UNIT') == 'in' ? 'inch' : Configuration::get('PS_DIMENSION_UNIT');
        $weightUnit = Configuration::get('PS_WEIGHT_UNIT') == 'kg' ? 'kgs' : Configuration::get('PS_WEIGHT_UNIT');
        // Instant PackageDimension class
        $packageDimension = new Bean\PackageDimension($dimensionUnit, $weightUnit);

        $packageSettingType = Configuration::get('UPS_PKG_SETTING_TYPE');
        $listPackages = [];
        if ($packageSettingType == false || $packageSettingType == $packageDimension::TYPE_PACKAGE_DEFAULT) {
        // Get list default package
                $listDefaultPackage = unserialize(Configuration::get('UPS_PKG_DEFAULT'));
            //  $listDefaultPackage = $this->getDefaultPackage(); //old code

             $numberOfCartItem = 0;
            foreach ($products as $key => $prod) {
                if(isset($prod['product_quantity'])) {
                $numberOfCartItem += isset($prod['product_quantity']) ? $prod['product_quantity'] : 0;
             } elseif ($prod['cart_quantity']) {
                $numberOfCartItem += isset($prod['cart_quantity']) ? $prod['cart_quantity'] : 0;
             }
            }

        // Set package dimension (case package default)
             $packageDimension->setDefaultPackageDimension($listDefaultPackage, $numberOfCartItem);

             for ($i = 1; $i <= $packageDimension->numberOfPackage; $i++) {
                $package = [];
                $package['length'] = (string)$packageDimension->length;
                $package['width'] = (string)$packageDimension->width;
                $package['height'] = (string)$packageDimension->height;
                $package['unit_dimension'] = $packageDimension->dimensionUnit;
                $package['weight'] = (string)$packageDimension->weight;
                $package['unit_weight'] = $packageDimension->weightUnit;
                $listPackages[] = $package;
            }
        } elseif ($packageSettingType == $packageDimension::TYPE_PRODUCT_DIMENSION) {
            $selected_pac_algo = Configuration::get('UPS_PKG_ALGO_TYPE');
            if (empty($selected_pac_algo)) {
                $selected_pac_algo = 3;
            }
    
            $cartProduct = $products;
            if ($selected_pac_algo == 1 || $selected_pac_algo == 2) {
                $pack_weg_unit = Configuration::get('UPS_PKG_WEIG_UNIT');
                if ($selected_pac_algo == 2) { //Weight Based  Packing
                    $pack_max_weight = Configuration::get('UPS_PKG_MAX_WEG');
                    $listPackages = $packageDimension->weight_pack($cartProduct, $pack_weg_unit, $pack_max_weight);
                } else { //Pack Item Indidividually
                $pack_dim_unit = Configuration::get('UPS_PKG_DIM_UNIT');
                $listPackages = $packageDimension->per_item($cartProduct, $pack_weg_unit, $pack_dim_unit);
                }
            } elseif ($selected_pac_algo == 3) { //Box Based Packing
                
                $listProductDimension = unserialize(Configuration::get('UPS_PKG_PRODUCT_DIMENSION'));
                $listPackages = $packageDimension->box_pack($cartProduct, $listProductDimension);
               
            }
        }

        $ap_name          = "";
        $ap_id            = "";
        $ap_address1      = "";
        $ap_address2      = "";
        $ap_city          = "";
        $ap_postcode      = "";
        $ap_state         = "";
        $shippingService  = "";
        $arrAccessorial   = []; 

        Db::getInstance()->insert(
            Constants::DB_TABLE_OPENORDER,
            array(
                array(
                    'id_order'             => (int) $order_id,
                    'ap_id'                => pSQL($ap_id),
                    'ap_name'              => pSQL($ap_name),
                    'ap_address1'          => pSQL($ap_address1),
                    'ap_address2'          => pSQL($ap_address2),
                    'ap_city'              => pSQL($ap_city),
                    'ap_postcode'          => pSQL($ap_postcode),
                    'ap_state'             => pSQL($ap_state),
                    'shipping_service'     => pSQL($shippingService),
                    'accessorials_service' => pSQL(serialize($arrAccessorial)),
                    'package'              => pSQL(serialize($listPackages)),
                ),
            )
        );
    }
    // Get orders to migrate
    public function getOrdersToMigrate()
    {
        $sql = new DbQuery();
        $sql->select('o.id_order as id');
        $sql->leftJoin(Constants::DB_TABLE_OPENORDER, 'op', 'op.id_order = o.id_order');
        $sql->from('orders', 'o');
        $sql->where("op.id_order IS NULL AND o.current_state IN ("
            . Constants::STATUS_PROCESSING_IN_PROGRESS .
            ")");

        return Db::getInstance()->executeS($sql);
    }

    public function processDownloadRegis()
    {
        $iso_code = $this->context->language->iso_code;
        $fileCodForm = PATH_ASSETS_FOLDER . Constants::NAME_PICKUP_PU_FORM . $iso_code . '.pdf';

        if (file_exists($fileCodForm)) {
            $this->getHeader($fileCodForm);
        } else {
            $fileCodForm = PATH_ASSETS_FOLDER . Constants::NAME_PICKUP_PU_FORM . 'pl.pdf';
            if (file_exists($fileCodForm)) {
                $this->getHeader($fileCodForm);
            }
        }
    }

    public function processDownloadCod()
    {
        $iso_code = $this->context->language->iso_code;
        $fileCodForm = PATH_ASSETS_FOLDER . Constants::NAME_COD_PU_FORM . $iso_code . '.pdf';

        if (file_exists($fileCodForm)) {
            $this->getHeader($fileCodForm);
        } else {
            $fileCodForm = PATH_ASSETS_FOLDER . Constants::NAME_COD_PU_FORM . 'pl.pdf';
            if (file_exists($fileCodForm)) {
                $this->getHeader($fileCodForm);
            }
        }
    }

    public function getHeader($fileCodForm)
    {
        header('Content-Description: File Transfer');
        header('Content-Type: application/octet-stream');
        header('Content-Disposition: attachment; filename="' . basename($fileCodForm) . '"');
        header('Expires: 0');
        header('Cache-Control: must-revalidate');
        header('Pragma: public');
        header('Content-Length: ' . filesize($fileCodForm));
        readfile($fileCodForm);
    }

    private function getScheduleTexts()
    {
        switch ($this->module->pluginCountryCode) {
            case Constants::POLAND_ISO:
                return $this->sdk->t('billing', 'txtInfoPL');
            case Constants::UNITED_STATE_ISO:
                return $this->sdk->t('billing', 'txtInfoUS');
            default:
                return $this->sdk->t(
                    'billing',
                    'txtInfo_NotForPL',
                    array('contact_page' => $this->links['contact_page'])
                );
        }
    }

    private function getFQAlinks()
    {
        switch ($this->module->pluginCountryCode) {
            case Constants::POLAND_ISO:
                $txt1 = $this->sdk->t('billing', 'txtPickupElement4PL');
                $txt2 = $this->sdk->t('billing', 'txtPickupElement4PL_sub');
                return "<p class='pad-paragraph'>$txt1</p><p class='pad-paragraph'>$txt2</p>";
            case Constants::UNITED_STATE_ISO:
                $txt1 = $this->sdk->t('billing', 'txtPickupElement4_NotForPL');
                $txt2 = $this->sdk->t('billing', 'txtPickupElement4US');
                return "<p class='pad-paragraph'>$txt1</p><p class='pad-paragraph'>$txt2</p>";
            
            default:
                $txt1 = $this->sdk->t('billing', 'txtPickupElement4_NotForPL');
                $txt2 = $this->sdk->t(
                    'billing',
                    'txtPickupElement4_NotForPL_sub',
                    array('fqa_link' => $this->links['contact_page'])
                );
                return "<p class='pad-paragraph'>$txt1</p><p class='pad-paragraph'>$txt2</p>";
        }
    }

    private function generateBtnPrintForm($link)
    {
        switch ($this->module->pluginCountryCode) {
            case Constants::POLAND_ISO:
                return '<p class="pad-paragraph"><a class="btn btn-default" href="' . $link . '">'
                    . $this->sdk->t('billing', 'txtPrintForm') . '</a></p>';
            
            default:
                return '';
        }
    }

    private function createCarrier()
    {
        $langDefault = Configuration::get('PS_LANG_DEFAULT');

        $carrier                       = new Carrier();
        $carrier->name                 = 'UPS SHIPPING';
        $carrier->active               = Module::isEnabled($this->module->name);
        $carrier->deleted              = 0;
        $carrier->shipping_handling    = false;
        $carrier->range_behavior       = 0;
        $carrier->shipping_method      = Carrier::SHIPPING_METHOD_PRICE;
        $carrier->delay[$langDefault]  = '  '; //Need handle by APIs
        $carrier->shipping_external    = true;
        $carrier->is_module            = true;
        $carrier->external_module_name = $this->module->name;
        $carrier->need_range           = true;
        $carrier->is_free              = false;
        $carrier->url                  ='https://www.ups.com/track?loc=en_US&tracknum=@';

        if ($carrier->add()) {
            // Set Gruops for carrier
            $groups = Group::getGroups(true);
            $arrGroupId = array_column($groups, 'id_group');
            $carrier->setGroups($arrGroupId);

            // Set Zones for carrier
            $zones = Zone::getZones(true);
            foreach ($zones as $zone) {
                Db::getInstance()->insert(
                    'carrier_zone',
                    array(
                        'id_carrier' => (int) $carrier->id,
                        'id_zone'    => (int) $zone['id_zone'],
                    )
                );
            }

            // Add Range Price
            $rangePrice             = new RangePrice();
            $rangePrice->id_carrier = $carrier->id;
            $rangePrice->delimiter1 = '0';
            $rangePrice->delimiter2 = '1000000';
            $rangePrice->add();

            // Add logo for carrier
            $pathLogoModule = _PS_MODULE_DIR_ . $this->module->name . '/views/img/' . 'ups_logo_40.jpg';
            $pathShipImg = _PS_SHIP_IMG_DIR_ . '/' . (int) $carrier->id . '.jpg';
            copy($pathLogoModule, $pathShipImg);

            Configuration::updateValue('PS_CARRIER_DEFAULT', $carrier->id);
            Configuration::updateValue('UPS_SHIPING_METHOD_ID', $carrier->id);
            Configuration::updateValue('UPS_SHIPING_METHOD_REFERENCE_ID', $carrier->id);
        }

        return true;
    }
}
?>