<?php

namespace Ups\Api;

/**
 * Created by UPS
 * Created at 19/06/2018
 */

use Configuration;
use Ups\Api\CommonHandle;

class LabelRecovery extends CommonHandle
{
    const PATH = 'LBRecovery';
    protected $_reference;
    public function __invoke($args = array())
    {
        if ($this->e == null) {
            $this->e = new \Exception;
        }

        $this->path = SELF::PATH;
        return parent::__invoke($args);
    }

    protected function responseHandler($response)
    {
        $data = parent::responseHandler($response);
        $res = $data->LabelRecoveryResponse;

        if (isset($res)) {
            $return = [
                'Code' => $res->Response->ResponseStatus->Code,
                'Description' => $res->Response->ResponseStatus->Description,
                'LabelResults' => $res->LabelResults
            ];
        } elseif (isset($data->Fault)) {
            $return = [
                'Code' => $data
                    ->Fault
                    ->detail
                    ->Errors
                    ->ErrorDetail
                    ->PrimaryErrorCode
                    ->Code,
                'Description' => $data
                    ->Fault
                    ->detail
                    ->Errors
                    ->ErrorDetail
                    ->PrimaryErrorCode
                    ->Description
            ];
        } else {
            $return = [
                'Code' => -1,
                'Description' => 'No internet connection'
            ];
        }

        return $return;
    }

    protected function resolveParam($args)
    {
        $ref_config = \Configuration::get("UPS_SP_SERV_SET_ORDER_REF_NUM");

        
        $ref_val = '';
        if ($ref_config) {
            $sql = "SELECT o.reference FROM `" . _DB_PREFIX_ . "orders` as o LEFT JOIN `" . _DB_PREFIX_ . "ups_shipment` as s ON o.id_order=s.id_order WHERE tracking_number='" . $args['trackingNumber'] . "'";
            $ref_val = \Db::getInstance()->getValue($sql);
        }
        $data = [
            'LabelRecoveryRequest' => [
                'RequestOption' => [
                    'SubVersion' => '1701',
                ],
                'LabelSpecification' => [
                    'LabelImageFormat' => [
                        'Code' => $args['labelFormat'],
                    ],
                    'HTTPUserAgent' => 'Mozilla/4.5',
                ],
                'Translate' => [
                    'LanguageCode' => 'eng',
                    'DialectCode' => 'GB',
                    'Code' => '01',
                ],
                'TrackingNumber' => $args['trackingNumber'],
                'ReferenceValues' => [
                    'ReferenceNumber' => [
                        'Value' => $ref_val,
                    ],
                ]
            ]
        ];

        parent::resolveParam($data);
    }
}
