<?php

namespace Ups\Api;

use Controller;

/**
 * Trait for Shipment Service Options and Package Service Options
 * Created by UPS
 * Created at 26-08-2018
 */

trait ServiceOptionsTrait
{
    public $country;
    public $language;
    public $currency;
    public $orderValue;

    public $shipmentLvlSupportedCountries = ['PL', 'GB'];
    public $packageLvlSupportedCountries = ['GB', 'US'];

    public $listCountryLanguages = [
        'CZ'=>'CES',
        'DK'=>'DAN',
        'DE'=>'DEU',
        'GR'=>'ELL',
        'EN'=>'ENG',
        'FI'=>'FIN',
        'FR'=>'FRA',
        'BE'=>'FRA',
        'HU'=>'HUN',
        'IT'=>'ITA',
        'NL'=>'NLD',
        'NO'=>'NOR',
        'PL'=>'POL',
        'RO'=>'RON',
        'RU'=>'RUS',
        'SK'=>'SLK',
        'ES'=>'SPA',
        'SE'=>'SWE',
        'TR'=>'TUR',
        'AT'=>'AUT',
        'BG'=>'BGR',
        'HR'=>'BIH',
        'CY'=>'CYP',
        'EE'=>'EST',
        'IE'=>'IRL',
        'LV'=>'LVA',
        'LT'=>'LTU',
        'LU'=>'LUX',
        'MT'=>'MLT',
        'SI'=>'SVN',
        'CH'=>'GSW',
        'RS'=>'SRP',
        'NO'=>'NOB',
        'IS'=>'ISL',
        'JE'=>'JEY',
        'TR'=>'TUR',
    ];
    public $listCountryDialects = [
        'CZ',
        'DK',
        'BE',
        'DE',
        'GR',
        'FI',
        'FR',
        'HU',
        'IT',
        'NL',
        'NO',
        'PL',
        'RU',
        'SK',
        'ES',
        'SE',
        'TR',
        'AT',
        'BG',
        'HR',
        'CY',
        'EE',
        'IE',
        'LV',
        'LT',
        'LU',
        'MT',
        'RO',
        'SI',
        'RS',
        'NO',
        'CH',
        'IS',
        'JE',
        'TR'
    ];

    // List Shipping Service Options DOESN'T EXISTS in Rate API.
    public $ignorList = ['012','6','372'];

    public $countryCodeEu = array(
        'BE', 'NL', 'FR', 'ES', 'PL', 'IT', 'DE', 'GB', 'AT', 'BG', 'HR', 'CY', 'CZ', 'DK', 'EE', 'FI', 'GR', 'HU', 'IE', 'LV', 'LT', 'LU', 'MT', 'RO', 'SK', 'SI', 'SE', 'RS', 'NO', 'CH', 'IS', 'JE', 'TR'
    );

    public function packageHandle($packages, $accessorials, $country = null)
    {
        $packageData = [];

        if (empty($packages))
        {
            return $packageData;
        }

        // Get Package Service Option.
        // Have to check empty 8:46 (26-08-2018 UPS)
        $accessorialOptions = $this->packageServiceOptionsHandle($accessorials, $country);

        foreach ($packages as $package)
        {
            $unitLength = isset($package['unit_dimension']) ? trim(strtoupper($package['unit_dimension'])) : "";
            if ('INCH' == $unitLength) {
                $unitLength = 'IN';
            }
            if ((isset($package['length']) && isset($package['width']) && isset($package['height']))) {
            $tmp["Dimensions"] = [
                "UnitOfMeasurement" => [
                    "Code" => $unitLength,
                    "Description" => strtoupper($package['unit_dimension']),
                ],
                "Length" => $package['length'],
                "Width" => $package['width'],
                "Height" => $package['height']
            ];
            }
            $tmp["PackageWeight"] = [
                "UnitOfMeasurement" => [
                    "Code" => strtoupper($package['unit_weight']),
                    "Description" => strtoupper($package['unit_weight']),
                ],
               "Weight" => $package['weight']
            ];

            $tmp["Packaging"] = ["Code" => "02"];
            $tmp["PackagingType"] = ["Code" => "02"];

            $packageData[] = array_merge($tmp, $accessorialOptions);
        }

        return $packageData;
    }

    /**
     * ACCESSORIAL SERVICE - Shipment Service Options
     *
     * 1. To AP COD
     * 2. Saturday Delivery
     * 3. Carbon Neutural
     * 4. Direct Delivery
     *
     * Updated at 26-08-2018 by UPS
     */
    public function shipmentServiceOptionsHandle(
        $accessorials,
        $order,
        $country = null,
        $api = '',
        $countryCodeEshoper = ''
    ) {
        $services = [];

        if (empty($accessorials))
        {
            return $services;
        }

        /**
         * Required when use Ship To Access Point
         * Add Notification for Alternate Delivery Location
         */
        if (!empty($order))
        {
            $this->country = $order['country_code'];

            if ($order['ap_city'] !== '')
            {
                $accessorials[] = '012';
            }
        }
        else
        {
            $this->country = $countryCodeEshoper;
        }

        $locale = $this->mappingDialectLanguage($countryCodeEshoper);

        foreach ($accessorials as $code)
        {
            if ($api === 'Rate')
            {
                if (array_search($code, $this->ignorList) !== false)
                {
                    continue;
                }
            }

            switch ($code)
            {
                /**
                 * Shipment notification
                 *
                 * 012 - Alternate Delivery Location Notification
                 *
                 * 1 .Notification Language - Page 67
                 * 2. Language / Dialect Combinations - Page 431
                 *
                 */
                case '012':
                    $services['Notification'][] = [
                        'NotificationCode' => '012',
                        'EMail' => [
                            'EMailAddress' => $order['email']
                        ],
                        'Locale' => [
                            'Language' => $locale['Language'],
                            'Dialect' => $locale['Dialect']
                        ]
                    ];
                break;


                /**
                 * Shipment Service Option
                 *
                 * Access Point COD Indicates. Package COD is requested for a shipment.
                 * Valid only for 01 - Hold For Pickup at UPS AP
                 *
                 * Page 65?
                 */
                case '4':
                    $services = array_merge($services, $this->codServiceOption('AP', $country));
                break;


                case '500':
                    $services = array_merge($services, $this->codServiceOption('Home', $country));
                break;


                /**
                 * Quantum view Ship Notification
                 * Notification Codes: 6
                 * Page 66
                 */
                case '6':
                    $services['Notification'][] = [
                        'NotificationCode' => '6',
                        'EMail' => [
                            'EMailAddress' => $order['email']
                        ],
                        'Locale' => [
                            'Language' => $locale['Language'],
                            'Dialect' => $locale['Dialect']
                        ]
                    ];
                break;

                /**
                 * Paperless Invoice
                 * updated 2022
                 */

                case '022':
                    
                    $order_details = \OrderDetail::getList($order['id_order']);

                    //$address_delivery = isset($order['address_delivery1']) && isset($order['address_delivery2']) && !empty($order['address_delivery2']) ? $order['address_delivery1'].", ".$order['address_delivery2'] : $order['address_delivery1'] ;
                    $address_delivery1 = (isset($order['address_delivery1'])) ? $order['address_delivery1'] : '';
                    $address_delivery2 = (isset($order['address_delivery2']) && !empty($order['address_delivery2'])) ? $order['address_delivery2'] : '';

                    $services['AdditionalDocumentIndicator'][] = '1';
                    
                    $services['InternationalForms'][] = [
                        'FormType' => '01',
                        'InvoiceDate' => date("Ymd"),
                        'Contacts' => [
                            'SoldTo' => [
                                'Name' => $order['firstname'],
                                'AttentionName' => isset($order['lastname'])&& !empty($order['lastname']) ? $order['lastname'] : $order['firstname'],
                                'Number' => isset($order['phone']) ? $order['phone'] : '',
                                'Address' => [
                                    'AddressLine' => [
                                        substr($address_delivery1, 0, 35),
                                        substr($address_delivery2, 0, 35),
                                    ],
                                    'City' => isset($order['city']) ? $order['city'] : '',
                                    'StateProvinceCode' => isset($order['state_code']) ? $order['state_code'] : '',
                                    'PostalCode' => isset($order['postcode']) ? $order['postcode'] : '',
                                    'CountryCode' => isset($order['country_code']) ? $order['country_code'] : '',
                                    'EMailAddress' => isset($order['email']) ? $order['email'] : '',
                                ],
                            ]
                            
                            ],
                            'Product' => [],
                            'ReasonForExport' => 'SALE',
                            'CurrencyCode' => $order['currency']
                    ];

                    foreach($order_details as $prods){
                        $prod['Product'][] = [
                            'Description' => substr((string)$prods['product_name'],0,35),
                            'Unit' => [
                                'Number' => $prods['product_quantity'],
                                'UnitOfMeasurement' => [
                                    'Code' => 'KG',
                                    'Description' => 'Kilogram'
                                ],
                                'Value' => (string)round($prods['product_price']+$prods['product_tax'],2)
                            ],
                            'OriginCountryCode'=> $country->ShipFromCountryCode,
                            'CommodityCode'=> \Configuration::get('product_commodity_'.$prods['product_id'])
                        ];
                    }
                        
                        foreach($services['InternationalForms'] as $key=>$val){
                           foreach($val as $ky=>$v){
                            if($ky == 'Product'){
                                $services['InternationalForms'][$key]['Product'] = $prod['Product'];
                            }
                           }
                            
                        }
                    //    $services['InternationalForms']['Product'] =  $prod['Product'];
                        
                break;


                /**
                 * Quantum view notify delivery (Shipment Service Option)
                 *
                 * Was contained in Quantum View Notification (QVN)
                 * Notification Codes:
                 * 5 - QV In-transit Notification
                 * 6 - QV Ship Notification
                 * 7 - QV Exception Notification
                 * 8 - QV Delivery Notification
                 * 2 - Return Notification or Label Creation Notification
                 * 012 - Alternate Delivery Location Notification
                 * 013 - UAP Shipper Notification
                 *
                 * Page 66
                 */
                case '372':
                    $services['Notification'][] = [
                        'NotificationCode' => '8',
                        'EMail' => [
                            'EMailAddress' => $order['email']
                        ],
                        'Locale' => [
                            'Language' => $locale['Language'],
                            'Dialect' => $locale['Dialect']
                        ]
                    ];
                break;


                /**
                 * Residential Address Indicator
                 *
                 * This field is a flag to indicate
                 * if the receiver is a residential location.
                 *
                 * Page 44
                 */
                case '270':
                    // ['ShipTo']['Address']
                    $this->addFlag('ResidentialAddressIndicator');

                break;


                /**
                 * Saturday Delivery indicator.
                 * This is an empty tag, any value inside is ignored.
                 * Page 64.
                 */
                case '300':
                    $indicator = '';
                    if (!empty($country) && in_array(strtolower($country->ShipToCountryCode), array('us'))) {
                        $indicator = '1';
                    }
                    $services['SaturdayDeliveryIndicator'] = $indicator;
                break;


                /**
                 * Carbon Neutral
                 * Page 96
                 */
                case '441':
                    $services['UPScarbonneutralIndicator'] = '';
                break;


                /**
                 * Direct Delivery Only
                 * Any value inside is ignored.
                 *
                 * This accessorial is not valid with Shipment Indicatoion Type:
                 * 01 - Hold For Pickup at UPS AP and
                 * 02 - UPS AP Delivery
                 * Page 66
                 */
                case '541':
                    $services['DirectDeliveryOnlyIndicator'] = '';
                break;


                 /**
                 * Delivery Confirmation Signature Required.
                 * Page 96.
                 */
                case '2':
                    if (
                        !empty($country) &&
                        ($country->ShipFromCountryCode != 'US' || $country->ShipToCountryCode != 'US')
                    ) {
                        $services['DeliveryConfirmation']['DCISType'] = '1';
                    }
                break;


                /**
                 * Delivery Confirmation Adult Signature Required.
                 * Page 96.
                 */
                case '3':
                    if (
                        !empty($country) &&
                        ($country->ShipFromCountryCode != 'US' || $country->ShipToCountryCode != 'US')
                    ) {
                        $services['DeliveryConfirmation']['DCISType'] = '2';
                    }
                break;

                default:

                break;
            }
        }

        return $services;
    }

    /**
     * ACCESSORIAL SERVICE - Package Service Options
     *
     * 1. Delivery Confirmation Signature Required.
     * 2. Delivery Confirmation Adult Signature Required.
     * 3. To AP COD.
     * 4. Declared Value.
     * 5. Additional Handling Indicator.
     *
     * Updated at 26-08-2018 by UPS
     */
    public function packageServiceOptionsHandle($accessorials, $country)
    {
        $accessorialOptions = array();

        foreach ($accessorials as $code)
        {
            switch ($code)
            {
                /**
                 * Declared Value
                 * Page 105
                 */
                case '5':
                    $accessorialOptions['PackageServiceOptions']['DeclaredValue'] = [
                        'CurrencyCode' => $this->currency,
                        'MonetaryValue' => $this->orderValue
                    ];

                break;

                /**
                 * Additional Handling Indicator
                 * Page 104
                 */
                case '100':
                    $accessorialOptions['AdditionalHandlingIndicator'] = 'true';
                break;

                case '2':
                    if (
                        !empty($country) &&
                        ($country->ShipToCountryCode == 'US' && $country->ShipFromCountryCode == 'US')
                    ) {
                        $accessorialOptions['PackageServiceOptions']['DeliveryConfirmation']['DCISType'] = '2';
                    }
                break;

                case '3':
                    if (
                        !empty($country) &&
                        ($country->ShipToCountryCode == 'US' && $country->ShipFromCountryCode == 'US')
                    ) {
                        $accessorialOptions['PackageServiceOptions']['DeliveryConfirmation']['DCISType'] = '3';
                    }
                break;
                case '500':
                    if (!empty($country) &&
                        in_array(strtolower($country->ShipToCountryCode), array('us', 'ca', 'mx', 'pr'))) {
                        $accessorialOptions['PackageServiceOptions']['COD'] = [
                            'CODFundsCode' => "0",
                                'CODAmount' => [
                                    'CurrencyCode' => $this->currency,
                                    'MonetaryValue' => $this->orderValue
                                ]
                        ];
                    }
                break;

                default:

                break;

            }
        }

        return $accessorialOptions;
    }

    public function addFlag($flag)
    {
        $this->flag[] = $flag;
    }

    public function findKey($array, $keySearch)
    {
        foreach ($array as $key => $item)
        {
            if ($key == $keySearch)
            {
                return true;
            }
            elseif (is_array($item) && $this->findKey($item, $keySearch))
            {
                return true;
            }
        }
        return false;
    }

    public function mappingDialectLanguage($country)
    {
        $countryCodeUpper = strtoupper($country);
        //language
        $selectedLanguage = 'ENG';
        $arrayKey = array_keys($this->listCountryLanguages);
        if (in_array($countryCodeUpper, $arrayKey)) {
            $selectedLanguage = $this->listCountryLanguages[$countryCodeUpper];
        }
        // dialect
        $selectedDialect = 'GB';
        if (in_array($countryCodeUpper, $this->listCountryDialects)) {
            $selectedDialect = '97';
        }
        if ('RO' == $countryCodeUpper) {
            $selectedDialect = 'RO';
        }
        return [
            'Language' => $selectedLanguage,
            'Dialect' => $selectedDialect
        ];
    }

    /**
     * Shipment & Package Service Option
     * Access Point COD Indicates. Package COD is requested for a shipment.
     * Valid only for 01 - Hold For Pickup at UPS AP
     * Page 106 65?
     */
    public function codServiceOption($to, $country = null)
    {
        $serviceOptions = [];

        switch ($to)
        {
            case 'AP':
                $serviceOptions = [
                    'AccessPointCOD' => [
                        'CurrencyCode' => $this->currency,
                        'MonetaryValue' => $this->orderValue
                    ]
                ];
            break;

            case 'Home':
                if (!empty($country) &&
                    !in_array(strtolower($country->ShipToCountryCode), array('us', 'ca', 'mx', 'pr'))) {
                    $serviceOptions = [
                        'COD' => [
                            'CODFundsCode' => "1",
                            'CODAmount' => [
                                'CurrencyCode' => $this->currency,
                                'MonetaryValue' => $this->orderValue
                            ]
                        ]
                    ];
                }
            break;
        }

        return $serviceOptions;
    }

    public function checkSupportedCountries($level)
    {
        $code = false;

        if ($level == static::SHIPMENT_LEVEL)
        {
            if (array_search($this->country, $this->shipmentLvlSupportedCountries) !== false)
            {
                $code = '1';
            }
        }
        elseif ($level == static::PACKAGE_LEVEL)
        {
            // No EU countries currently support Package level COD
            if (array_search($this->country, $this->packageLvlSupportedCountries) !== false)
            {
                $code = '0';
            }
        }

        return $code;
    }

    /**
     * Hold For Pickup at UPS AP.
     * Becareful with Shipping Service Option below. (Direct Delivery Only)
     *
     * Updated at 26-08-2018 / 24-09-2018 by UPS
     */
    public function checkShipmentType($order)
    {
        $shipmentTypes = [];
        if (!empty($order) && isset($order['ap_city']) && $order['ap_city'] !== '')
        {
            $apInfo = [
                'ApName' => $order['ap_name'],
                'firstname' => $order['firstname'],
                'lastname' => $order['lastname'],
                'ApAddressLine' => $order['ap_address1'],
                'ApCity' => $order['ap_city'],
                'ApStateProvinceCode' => $order['ap_state'],  //province_code -> sai
                'ApPostalCode' => $order['ap_postcode'],
                'ApCountryCode' => $order['country_code'],
            ];

            $shipmentTypes = $this->checkDeliveryAddressAp($apInfo);
        }

        return $shipmentTypes;
    }

    public function checkDeliveryAddressAp($apAddress)
    {
        $indication_type = isset($apAddress['ShipmentIndicationType']) ? $apAddress['ShipmentIndicationType'] : "01";
        $shipment_indication_type = '';
        if($indication_type == 02){
            $shipment_indication_type = '02';
        }else{
            $shipment_indication_type = '01';
        }
        //JIRA UAT 927
        $listCountry = ['US', 'CA', 'IE'];
        $stateProvinceCode = $apAddress['ApStateProvinceCode'];
        if (!in_array($apAddress['ApCountryCode'], $listCountry)) {
            $stateProvinceCode = 'XX';
        }
        //end jira

        $shipmentTypes = [
            'ShipmentIndicationType'   => ["Code" => $shipment_indication_type],
            'AlternateDeliveryAddress' => [
                'Name'    => $apAddress['ApName'],
                'Address' => [
                    'AddressLine'       => substr(mb_convert_encoding($apAddress['ApAddressLine'], 'UTF-8', 'HTML-ENTITIES'),0,35),
                    'City'              => $apAddress['ApCity'],
                    'StateProvinceCode' => $stateProvinceCode,
                    'PostalCode'        => preg_replace('/[^a-zA-Z0-9]/s','',$apAddress['ApPostalCode']),
                    'CountryCode'       => $apAddress['ApCountryCode'],
                ]
            ]
        ];

        if (isset($apAddress['firstname']) && isset($apAddress['lastname'])) {
            $shipmentTypes['AlternateDeliveryAddress']['AttentionName'] =
                $apAddress['firstname'] . ' ' . $apAddress['lastname'];
        }

        return $shipmentTypes;
    }

    public function checkShippingService($requestOption, $shippingServiceCode, $shippingServiceDesc)
    {
        $shippingService = array();

        if (
            $requestOption !== 'SHOP' &&
            strtolower($requestOption) !== 'shoptimeintransit' &&
            !empty($shippingServiceCode)
        ) {
            $shippingService = array(
                'Code' => $shippingServiceCode,
                'Description' => $shippingServiceDesc,
            );
        }

        return $shippingService;
    }

    public function checkShipmentTotalWeight($args)
    {
        $shippmentWeight = array();
        if (
            ($args['RequestOption'] == 'RATETIMEINTRANSIT' || strtolower($args['RequestOption']) == 'shoptimeintransit')
            && isset($args['PackageWeightCode'])
            && isset($args['PackageWeightDescription'])
            && isset($args['PackageWeightWeight'])
        ) {
            $shippmentWeight = array(
                "UnitOfMeasurement" => [
                    "Code"=> $args['PackageWeightCode'],
                    "Description"=> $args['PackageWeightDescription'],
                ],
                "Weight"=> $args['PackageWeightWeight'],
            );
        }

        return $shippmentWeight;
    }

    public function checkInvoiceLineTotal($args)
    {
        $invoiceLineTotal = array();

        if (isset($args['CurrencyCode']) && isset($args['MonetaryValue']))
        {
            $invoiceLineTotal = array(
                'CurrencyCode' => $args['CurrencyCode'],
                'MonetaryValue' => $args['MonetaryValue'],
            );
        }

        return $invoiceLineTotal;
    }

    public function checkTimeInformation($args)
    {
        $deliveryTimeInfor = array();

        if (
            ($args['RequestOption'] == 'RATETIMEINTRANSIT' || strtolower($args['RequestOption']) == 'shoptimeintransit')
            && isset($args['PickupDate'])
        ) {
            $deliveryTimeInfor = array(
                'PackageBillType' => '03',
                'Pickup' => array(
                    'Date' => $args['PickupDate']
                )
            );
        }

        return $deliveryTimeInfor;
    }

    public function getPaymentInformation($accountNumber, $shiptoCountry, $checkDutiesTax = false)
    {
        $paymentInformation = [
            'ShipmentCharge' => [
                [
                    'Type' => '01',
                    'BillShipper' => ['AccountNumber' => $accountNumber]
                ]
            ]
        ];

        /*if ($this->hasDutiesTax($shiptoCountry) || $checkDutiesTax)
        {
            $paymentInformation['ShipmentCharge'] = array_merge(
                $paymentInformation['ShipmentCharge'],
                [
                    [
                        'Type' => '02',
                        'BillShipper' => ['AccountNumber' => $accountNumber]
                    ]
                ]
            );
        }*/

        return $paymentInformation;
    }

    public function hasDutiesTax($shiptoCountry)
    {
        return (!in_array($shiptoCountry, static::COUNTRY_EU)) ? true : false;
    }

    public function checkDutiesTax($apCountryCode, $isApCod)
    {
        $isEuCountry = !in_array($apCountryCode, $this->countryCodeEu);

        return ($isApCod && $isEuCountry);
    }
}
