<?php

namespace Ups\Api;

/**
 * Created by UPS
 * Created at 20/07/2018
 *
 * Updated at 28/08/2018
 */

use Configuration;
use Ups\Api\CommonHandle;

class Shipping extends CommonHandle
{
    const PATH = 'Ship';
    private $shipment;
    private $flag = [];
    private $plt_suppored = array("AX", "FI", "AS", "AD", "AR", "AU", "AT", "A2", "PT", "BH", "BD", "BB", "BY", "BE", "BT", "BA", "BN", "BG", "KH", "CA", "IC", "CL", "CN", "CO", "CK", "CR", "HR", "CY", "CZ", "DK", "EC", "SV", "EN", "GB", "EE", "FO", "FJ", "FR", "PF", "GE", "DE", "GI", "GR", "GL", "GU", "GT", "GG", "GY", "HT", "HO", "NL", "HN", "HK", "HU", "IS", "IN", "ID", "IE", "IL", "IT", "JM", "JP", "JE", "KE", "KI", "KR", "KV", "KO", "LA", "LV", "LI", "LT", "LU", "MO", "M3", "MY", "MT", "MH", "MX", "FM", "MC", "MN", "MA", "NC", "NZ", "NG", "NF", "NB", "MP", "NO", "PW", "PG", "PE", "PH", "PL", "PO", "PR", "RO", "RT", "RU", "SP", "WS", "SM", "SA", "SF", "RS", "SG", "SK", "SI", "SB", "ZA", "ES", "C3", "UV", "KN", "VL", "SR", "SE", "CH", "TA", "TW", "TH", "TL", "TI", "TO", "TU", "TR", "TV", "AE", "US", "UY", "VU", "VA", "VN", "WL", "WF", "YA");
    use ServiceOptionsTrait;

    public function __invoke($args = array())
    {
        $this->path = SELF::PATH;
        // Debug style $return
        $return = parent::__invoke($args);

        return $return;
    }

    protected function responseHandler($response)
    {
        $data = parent::responseHandler($response);

        if (is_null($data)) {
            return [
                'Code' => 102,
                'Description' => 'There are errors connecting to the UPS API servers. Please try again.'
            ];
        } elseif (isset($data->Fault) || !isset($data->ShipmentResponse) || empty($data->ShipmentResponse)) {
            return [
                'Code' => $data->Fault->detail->Errors->ErrorDetail->PrimaryErrorCode->Code,
                'Description' => $data->Fault->detail->Errors->ErrorDetail->PrimaryErrorCode->Description
            ];
        }

        if (isset($data->ShipmentResponse)) {
            $res = $data->ShipmentResponse;

            $code = $res->Response->ResponseStatus->Code;
            $msg = $res->Response->ResponseStatus->Description;
            $result = $res->ShipmentResults;
            $shipmentId = $result->ShipmentIdentificationNumber;
            $packageResult = $result->PackageResults;
            $shippingFee = $result->ShipmentCharges->TotalCharges->MonetaryValue;

            // UAT 697
            if (isset($result->NegotiatedRateCharges->TotalCharge->MonetaryValue)) {
                $shippingFee = $result->NegotiatedRateCharges->TotalCharge->MonetaryValue;
            }

            return [
                'Code'        => $code,
                'Description' => $msg,
                'PackageResult' => $packageResult,
                'ShipmentId' => $shipmentId,
                // 'OrderId'     => $orderId,
                // 'Country'     => $country,
                'ShippingFee' => $shippingFee
            ];
        }
    }

    protected function resolveParam($args)
    {

        $data = $this->publishedRateShipment($args);
        // Debug style for return data
        parent::resolveParam($data);
    }

    private function publishedRateShipment($args)
    {
        /**
         * Handle Shipment Service Options
         * Updated at 28-08-2018
         * Updated by UPS
         */
        $this->currency = $args['currency'];

        $this->orderValue = (string) $args['totalOrderValue'];

        $order = $args['firstOrder'];
        $shiptoCountry = $order['country_code'];
        $shipToStateCode = 'XX';
        if (in_array(strtolower($shiptoCountry), ['us', 'ca', 'ie'])) {
            $shipToStateCode = (!empty($order['state'])) ? $order['state'] : ((!empty($order['state_code'])) ? $order['state_code'] : '');
        }
        $accountInfo = $args['accountInfo'];
        $primaryInfo = $args['primaryInfo'];
        $accountNumber = $accountInfo['AccountNumber'];
        $shipFromCountryCode = $accountInfo['CountryCode'];
        $postalcode = preg_replace('/[^a-zA-Z0-9]/s', '', $accountInfo['PostalCode']);

        $countryCodeData = new \stdClass();
        $countryCodeData->ShipFromCountryCode = $shipFromCountryCode;
        $countryCodeData->ShipToCountryCode = $shiptoCountry;

        $apCountryCode = '';
        if (!empty($order) && isset($order['ap_city']) && $order['ap_city'] !== '') {
            $apCountryCode = $order['country_code'];
        }

        $ph_number = isset($order['phone']) ? $order['phone'] : '';
        if (empty($ph_number)) {
            $ph_number = isset($order['phone_mobile']) ? $order['phone_mobile'] : '';
        }

        $order_details = new \Order((int) $order['id_order']);
        $addres_details = isset($order_details->id_address_delivery) ? new \Address((int) $order_details->id_address_delivery) : '';
        $attention = $order['firstname'] . ' ' . $order['lastname'];
        if (isset($addres_details->company) && !empty($addres_details->company)) {
            $attention = $addres_details->company;
        }

        $order_d = \OrderDetail::getList($order['id_order']);
        $prod_name = '';
        foreach ($order_d as $prods) {
            $prod_name .= $prods['product_name'] . ', ';
        }
        $prod_name = substr(rtrim($prod_name, ', '),0,49);
        $shipping_desc = 'Description';
        $configured_ship_desc = Configuration::get('UPS_SP_SERV_SET_SHIP_DESC');

        if (isset($configured_ship_desc) && !empty($configured_ship_desc)) {
            $shipping_desc = $configured_ship_desc;
            $shipping_desc = str_replace('{OrderID}', $order['id_order'], $shipping_desc);
            $shipping_desc = str_replace('{RefNo}', $order_details->reference, $shipping_desc);
            $shipping_desc = str_replace('{ProductName}', $prod_name, $shipping_desc);
        }
        
        
        $request = [
            'ShipmentRequest' => [
                'Request' => [
                    'RequestOption' => 'validate',
                    'SubVersion' => '1801'
                ],
                'Shipment' => [
                    'Description' => $shipping_desc,
                    'Shipper' => [
                        'Name' => $primaryInfo['CustomerName'],
                        'AttentionName' => $primaryInfo['CompanyName'],
                        'ShipperNumber' => $accountInfo['AccountNumber'],
                        'Phone' => [
                            'Number' => $primaryInfo['PhoneNumber'],
                        ],
                        'Address' => [
                            'AddressLine' => [
                                substr($accountInfo['AddressLine1'], 0, 35),
                                substr($accountInfo['AddressLine2'], 0, 35),
                                substr($accountInfo['AddressLine3'], 0, 35),
                            ],
                            'City' => $accountInfo['City'],
                            'StateProvinceCode' => $accountInfo['ProvinceCode'],
                            'PostalCode' => $postalcode,
                            'CountryCode' => $shipFromCountryCode
                        ],
                    ],
                    'ShipTo' => [
                        'Name' => $order['firstname'] . ' ' . $order['lastname'],
                        'AttentionName' => $attention,
                        'Phone' => [
                            'Number' => $ph_number,
                        ],
                        'Address' => [
                            'AddressLine' => [
                                substr($order['address_delivery1'], 0, 35),
                                substr($order['address_delivery2'], 0, 35),
                                // $args['AddressLine3']
                            ],
                            'City' => $order['city'],
                            'StateProvinceCode' => $shipToStateCode,
                            'PostalCode' => $order['postcode'],
                            'CountryCode' => $shiptoCountry
                        ],
                    ],
                    'ShipFrom' => [
                        'Name' => $primaryInfo['CustomerName'],
                        'AttentionName' => $primaryInfo['CompanyName'],
                        'Phone' => [
                            'Number' => $primaryInfo['PhoneNumber']
                        ],
                        'Address' => [
                            'AddressLine' => [
                                substr($accountInfo['AddressLine1'], 0, 35),
                                substr($accountInfo['AddressLine2'], 0, 35),
                                substr($accountInfo['AddressLine3'], 0, 35),
                            ],
                            'City' => $accountInfo['City'],
                            'StateProvinceCode' => $accountInfo['ProvinceCode'],
                            'PostalCode' => $postalcode,
                            'CountryCode' => $shipFromCountryCode
                        ],
                    ],
                    'Service' => [
                        'Code' => $args['shippingServiceInfo']['Ratecode'],
                        'Description' => $args['shippingServiceInfo']['name']
                    ],
                    "ShipmentRatingOptions" => [
                        "NegotiatedRatesIndicator" => ""
                    ],
                    'LabelSpecification' => [
                        'LabelImageFormat' => [
                            'Code' => 'GIF',
                            'Description' => 'GIF'
                        ],
                        'HTTPUserAgent' => 'Mozilla/4.5'
                    ],
                ]
            ]
        ];

        $ref_config = \Configuration::get("UPS_SP_SERV_SET_ORDER_REF_NUM");

        $ref_val = '';
        
        if ($ref_config) {
            $sql = "SELECT reference FROM `" . _DB_PREFIX_ . "orders` WHERE id_order ='" . $order['id_order'] . "'";
            $ref_val = \Db::getInstance()->getValue($sql);
            $request['ShipmentRequest']['Shipment'] = array_merge($request['ShipmentRequest']['Shipment'],['ReferenceNumber' => 
                ['Value'=> $ref_val]
            ]);
        }
    
        $is_plt = Configuration::get('UPS_PLT_ENABLE');

        if (in_array($shiptoCountry, $this->plt_suppored) && $is_plt) {
            if (!in_array('022', $args['accessorialsService'])) {
                array_push($args['accessorialsService'], '022');
            }
        }
        $shipmentServices = $this->shipmentServiceOptionsHandle($args['accessorialsService'], $order, $countryCodeData, '', $shiptoCountry);
        $packageData = $this->packageHandle($args['packages'], $args['accessorialsService'], $countryCodeData);
        $shipmentTypes = $this->checkShipmentType($order);
        $hasDutiesTax = $this->checkDutiesTax($apCountryCode, $args['isApCod']);
        $paymentInformation = $this->getPaymentInformation($accountNumber, $shiptoCountry, $hasDutiesTax);
        $invoiceLineTotal = $this->checkInvoiceLineTotal($args);

        /**
         * ADD Payment inforamtion with and without Duties, Tax
         */
        $request['ShipmentRequest']['Shipment'] =
            array_merge(
                $request['ShipmentRequest']['Shipment'],
                ['PaymentInformation' => $paymentInformation]
            );

        /**
         * ADD Shipping Service Options
         */
        $request['ShipmentRequest']['Shipment'] =
            array_merge(
                $request['ShipmentRequest']['Shipment'],
                ['ShipmentServiceOptions' => $shipmentServices]
            );

        /**
         * ADD Package List and Package Service Options
         */
        $request['ShipmentRequest']['Shipment'] =
            array_merge(
                $request['ShipmentRequest']['Shipment'],
                ['Package' => $packageData]
            );

        /**
         * ADD Shipment Indicator Types
         */
        $request['ShipmentRequest']['Shipment'] =
            array_merge(
                $request['ShipmentRequest']['Shipment'],
                $shipmentTypes
            );

        if (!empty($invoiceLineTotal)) {
            $request['ShipmentRequest']['Shipment'] =
                array_merge(
                    $request['ShipmentRequest']['Shipment'],
                    ['InvoiceLineTotal' => $invoiceLineTotal]
                );
        }

        /**
         * Add more information
         */
        if (!empty($this->flag)) {
            foreach ($this->flag as $key) {
                switch ($key) {
                    case 'ResidentialAddressIndicator':
                        $request['ShipmentRequest']['Shipment']['ShipTo']['Address'] =
                            array_merge(
                                $request['ShipmentRequest']['Shipment']['ShipTo']['Address'],
                                ['ResidentialAddressIndicator' => '1']
                            );
                        break;
                }
            }
        }
        return $request;
    }
}
